/**
 * Module Menu Editiing
 */
(function($) {
  $(document).ready(function() {
    
    /**
     * Actions for later use
     */
    var editButton    = '#wpbd-edit-menu',
        saveButton    = '#wpbd-save-menu',
        restoreButton = '#wpbd-restore-menu';
    
    /**
     * Var for controlling drag and drop events
     */
    var adminMenu,
        adminBar;
    
    /**
     * Parse original order of menu items
     */
    var getId = /wpbd-id-(.[^ ]+)/;
    
    // Loop menu items
    $('#adminmenu > li').each(function() {
      
      // The menu item
      var $menu = $(this);
      
      // Original order
      var menuID = getId.exec($(this).attr('class'));
      var visibility = $(this).hasClass('wpbd-menu-hidden') ? 1 : 0;
      // console.log(menuID);
      
      // Add as a data attribute
      if (menuID !== null) {
        $menu.append('<input type="hidden" name="wpbd-menu[]" value="'+ menuID[1] +'">');
        
        // New Features in menu editting
        // @since 0.0.21
        // Append new fields
        $menu.append('<input type="text" class="wpbd-rename-field" name="wpbd-menus-edits['+ menuID[1] +'][rename]" value="">');
        $menu.append('<input type="hidden" name="wpbd-menus-edits['+ menuID[1] +'][hidden]" value="'+ visibility +'">');
        // Append troggle selector 
        $menu.append('<div class="wpbd-hide-menu wpbd-ignore" data-menu="'+ menuID[1] +'"><span class="dashicons dashicons-visibility"></span></div>');
      
      } else {
        // console.log($menu);
      }
      
    }); // end each;
    
    /**
     * Activate edition on clicking edit
     */
    $(document).on('click', editButton, function(e) {
      
      e.preventDefault();
      
      // Hide and Show buttons
      $(editButton).hide();
      $(saveButton).show();
      $(restoreButton).show();
      
      // On edditing
      $('#adminmenu').addClass('wpbd-editting');
      
      // Add edit classes to separator
      $('.wp-menu-separator').addClass('wpbd-editing-separator');
      
      // Attach drag and drop on elements
      adminMenu = adminMenuDrag();
      
    });
    
    /**
     * Saves menu
     */
    $(document).on('click', saveButton, function(e) {
      e.preventDefault();

      // Display loading div
      $(saveButton).find('.wpbd-default').hide(0, function() {
        $(saveButton).find('.wpbd-saving').show();
      });
      
      // Get the data we want to save
      var data = $('#adminmenu :input').serialize();
      
      // Saves
      sendRequest(data, 'save_custom_menu_order', function() {
        
        // we need to hide our loading and display a success alert
        $(saveButton).find('.wpbd-saving').hide(0, function() {
          $(saveButton).find('.wpbd-saved').show().parent().addClass('wpbd-menu-saved');
        });
        
        // delay the reset
        setTimeout(function() {
                 
          // Close menu
          closeEdit();
          
          // reset buttons
          $(saveButton).find('.wpbd-saved').hide(0, function() {
            $(saveButton).find('.wpbd-default').show().parent().removeClass('wpbd-menu-saved');
          });
          
        }, 3000);
        
      }); // end callback
      
    });
    
    /**
     * Restore menu
     */
    $(document).on('click', restoreButton, function(e) {
      e.preventDefault();
      
      // Restore
      sendRequest(false, 'restore_menu_order', function() {
        
        // Refresh the page
        location.reload();
            
        // Close menu
        closeEdit();
      
      });
      
    });
    
    
    /**
     * Our Drag and Drop do menu
     */
    var adminMenuDrag = function() {
      
      // Admin menu containers
      var adminMenuContainers = [
        document.querySelector('#adminmenu')
      ];

      // Install our drag and drop interface
      var runAdminMenu = dragula(adminMenuContainers, {

        // When moving, do somethig
        moves: function (el, container, handle) {
          return true; // elements are always draggable by default
        },

        // Whats to ignore and not move
        invalid: function (el, target) { // prevent buttons and anchor tags from starting a drag
          var $el = $(el);
          return $el.is('#collapse-menu') || $el.is('.wpbd-ignore');
        },

        // Where to past the element
        mirrorContainer: document.querySelector('#adminmenu'), // set the element that gets mirror elements appended

        // remove on dropping outside
        removeOnSpill: false,

      }); // end adminMenu;
      
      // return object
      return runAdminMenu;
      
    }; // end adminMenuDrag;
    
    
    /**
     * Close edditing state
     */
    var closeEdit = function () {
      
      // Hide and Show buttons
      $(editButton).show();
      $(saveButton).hide();
      $(restoreButton).hide();
      
      // On edditing
      $('#adminmenu').removeClass('wpbd-editting');
      
      // Add edit classes to separator
      $('.wp-menu-separator').removeClass('wpbd-editing-separator');
            
      // Attach drag and drop on elements
      adminMenu.destroy();
      
    }; // end closeEdit;
    
    //
    //
    // Save Menu request
    //
    //
    var sendRequest = function(data, action, callback) {
      
      // Send our request to whatever action we need
      var request = $.ajax({
        url: ajaxurl,
        data: data + '&action=' + action,
        dataType: 'json',
        context: $('#adminmenu')
      });
      
      request.done(function() {
        callback();
      });
      
    };
    
    /**
     *
     * Lets handle the visibility toggle in the menu now
     * @since 0.0.21
     *
     */
    $(document).on('click', '.wpbd-hide-menu', function(e) {
      e.preventDefault();
      
      // We need to toggle the value for the input on that fields
      var menu  = $(this).data('menu');
      var field = $('[name="wpbd-menus-edits['+ menu +'][hidden]"]');
      
      // Toggle the value of the field
      var currentValue = parseInt(field.val(), 10);
      var newVal = 1 - currentValue;
      console.log(newVal);
      // Sets news value
      field.val(newVal);
      
      // Toggle class of the element
      field.parents('.wpbd-menu-item').toggleClass('wpbd-menu-hidden');
      
    });
    
    // Admin bar containers
//    var adminBarContainers = [
//      document.querySelector('#wp-admin-bar-root-default'),
//      document.querySelector('#wp-admin-bar-top-secondary')
//    ];
//    
//    // Install our drag and drop interface
//    var adminBar = dragula(adminBarContainers, {
//      
//      // Whats to ignore and not move
//      invalid: function (el, target) { // prevent buttons and anchor tags from starting a drag
//        return false;
//      },
//      
//      // Where to past the element
//      mirrorContainer: document.querySelector('#wp-admin-bar-root-default'), // set the element that gets mirror elements appended
//      
//      // remove on dropping outside
//      removeOnSpill: true,
//      
//      // Orientation
//      direction: 'horizontal',
//      
//    }); // end adminBar;
    
  });
})(jQuery);