<?php
/**
 * WP Ultimo: European VAT main class.
 *
 * @package WP_Ultimo_VAT
 * @since 1.0.0
 */

// Exit if accessed directly
defined('ABSPATH') || exit;

/**
 * WP Ultimo: European VAT main class
 *
 * This class instantiates our dependencies and load the things
 * our plugin needs to run.
 *
 * @package WP_Ultimo_VAT
 * @since 1.0.0
 */
final class WP_Ultimo_VAT {

	use \WP_Ultimo_VAT\Traits\Singleton;

	/**
	 * Checks if WP Ultimo: European VAT was loaded or not.
	 *
	 * This is set to true when all the WP Ultimo: European VAT requirements are met.
	 *
	 * @since 1.0.0
	 * @var boolean
	 */
	protected $loaded = false;

	/**
	 * Version of the Plugin
	 *
	 * @var string
	 */
	public $version = '1.0.0';

	/**
	 * Loads the necessary components into the main class
	 *
	 * @since 1.0.0
	 * @return void
	 */
	public function init() {
		/*
		 * Loads the WP_Ultimo\Helper class.
		 */
		$this->helper = WP_Ultimo_VAT\Helper::get_instance();

		/*
		 * Helper Functions
		 */
		require_once $this->helper->path('inc/functions/helper.php');

		/*
		 * Set up the text-domain for translations
		 */
		$this->setup_textdomain();

		/*
		 * Check if the WP Ultimo: European VAT requirements are present.
		 *
		 * Everything we need to run our setup install needs top be loaded before this
		 * and have no dependencies outside of the classes loaded so far.
		 */
		if (WP_Ultimo_VAT\Requirements::met() === false) {

			return;

		} // end if;

		$this->loaded = true;

		/**
		 * Run the updater.
		 */
		WP_Ultimo_VAT\Updater::get_instance();

		/**
		 * Loads the main functionality
		 */
		WP_Ultimo_VAT\Tax\EU_Vat::get_instance();

		/**
		 * Triggers when all the dependencies were loaded
		 *
		 * Allows plugin developers to add new functionality. For example, support to new
		 * Hosting providers, etc.
		 *
		 * @since 1.0.0
		 */
		do_action('wp_ultimo_vat_load');

	} // end init;

	/**
	 * Returns true if all the requirements are met.
	 *
	 * @since 1.0.0
	 * @return boolean
	 */
	public function is_loaded() {

		return $this->loaded;

	} // end is_loaded;

	/**
	 * Setup the plugin text domain to be used in translations.
	 *
	 * @since 0.0.1
	 * @return void
	 */
	public function setup_textdomain() {
		/*
		 * Loads the translation files.
		 */
		load_plugin_textdomain('wp-ultimo-vat', false, dirname(WP_ULTIMO_VAT_PLUGIN_BASENAME) . '/lang');

	} // end setup_textdomain;

} // end class WP_Ultimo_VAT;
